<?php

namespace Drupal\cookiepro_plus;

use Drupal\cookiepro_plus\Event\CookieProGetDomainScript;
use Drupal\cookiepro_plus\Utility\ConfigHelper;
use Drupal\cookiepro_plus\Utility\CookieProHelperInterface;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\State\StateInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Contracts\EventDispatcher\EventDispatcherInterface;

/**
 * Class CookiePro.
 *
 * The main CookiePro service.
 */
class CookiePro implements CookieProInterface {

  /**
   * Cookie category ID labels, keyed by language code.
   *
   * @var \Drupal\Core\StringTranslation\TranslatableMarkup[][]
   */
  protected $categoryIdLabels;

  /**
   * Cookie category ID names, for no-consent messages, keyed by language code.
   *
   * @var \Drupal\Core\StringTranslation\TranslatableMarkup[][]
   */
  protected $categoryIdNames;

  /**
   * Cookie category ids.
   *
   * @var string[]
   */
  protected $categoryIds;

  /**
   * Configuration object.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * The current Request Stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The event dispatcher service.
   *
   * @var \Symfony\Contracts\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;

  /**
   * The CookiePro helper.
   *
   * @var \Drupal\cookiepro_plus\Utility\CookieProHelperInterface
   */
  protected $helper;

  /**
   * TRUE if the current path is excluded.
   *
   * @var bool
   */
  protected $isExcludedPath;

  /**
   * TRUE if the current path is in the limited to list.
   *
   * @var bool
   */
  protected $isLimitedToPath;

  /**
   * TRUE if the client IP(s) is (are) whitelisted for CookiePro bypass.
   *
   * @var bool
   */
  protected $isWhitelistedIp;

  /**
   * The logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The state keyvalue collection.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The domain.
   *
   * @var string
   */
  protected $domain;

  /**
   * The domain script for current mode.
   *
   * @var string
   */
  protected $domainScript;

  /**
   * IPv4 list that must bypass CookiePro.
   *
   * @var string[]
   */
  protected $ipv4Whitelist;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Configuration factory.
   * @param \Drupal\cookiepro_plus\Utility\CookieProHelperInterface $helper
   *   The CookiePro helper.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The current request stack.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger interface.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state keyvalue collection to use.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Symfony\Contracts\EventDispatcher\EventDispatcherInterface $event_dispatcher
   *   The event dispatcher service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    CookieProHelperInterface $helper,
    RequestStack $request_stack,
    LoggerInterface $logger,
    StateInterface $state,
    RendererInterface $renderer,
    EventDispatcherInterface $event_dispatcher
  ) {
    $this->config = $config_factory->get(static::CONFIG_KEY);
    $this->helper = $helper;
    $this->requestStack = $request_stack;
    $this->logger = $logger;
    $this->state = $state;
    $this->renderer = $renderer;
    $this->eventDispatcher = $event_dispatcher;
  }

  /**
   * {@inheritdoc}
   */
  public function addConfigCacheTags(array &$build): void {
    if (!isset($build['#cache']['tags'])) {
      $build['#cache']['tags'] = [];
    }
    $build['#cache']['tags'] = Cache::mergeTags($build['#cache']['tags'], $this->config->getCacheTags());
  }

  /**
   * {@inheritdoc}
   */
  public function disablePauseMode(): void {
    $this->setPauseMode();
  }

  /**
   * {@inheritdoc}
   */
  public function enablePauseMode(): void {
    $this->setPauseMode(TRUE);
  }

  /**
   * {@inheritdoc}
   */
  public function getAutoBlockScriptUrl(): string {
    return $this->getDomain() . sprintf(static::SCRIPT_PATH_AUTO_BLOCK, $this->getDomainScriptId());
  }

  /**
   * {@inheritdoc}
   */
  public function getCategoryCssClass(string $category_key): string {
    $category_id = $this->getCategoryIds()[$category_key] ?? NULL;
    return $category_id ? sprintf(static::COOKIE_CATEGORY_CLASS, $category_id) : '';
  }

  /**
   * {@inheritdoc}
   */
  public function getCategoryIdLabels(string $langcode = NULL): array {

    if (!isset($this->categoryIdLabels[$langcode])) {
      $key_labels = [
        static::CONF_CATEGORY_STRICTLY_NECESSARY => 'Strictly Necessary',
        static::CONF_CATEGORY_PERFORMANCE => 'Performance',
        static::CONF_CATEGORY_FUNCTIONAL => 'Functional',
        static::CONF_CATEGORY_TARGETING => 'Targeting',
        static::CONF_CATEGORY_SOCIAL_MEDIA => 'Social Media',
      ];

      // Translate with context to prevent collision with existing translations.
      foreach ($key_labels as $key => $name) {
        $this->categoryIdLabels[$langcode][$key] = $this->helper->translateWithContext($name, [], $langcode);
      }
    }

    return $this->categoryIdLabels[$langcode];
  }

  /**
   * {@inheritdoc}
   */
  public function getCategoryIds(): array {
    if (!isset($this->categoryIds)) {
      $this->categoryIds = $this->config->get(static::CONF_CATEGORY_IDS);
    }

    return $this->categoryIds;
  }

  /**
   * {@inheritdoc}
   */
  public function getCategoryIdNames(string $langcode = NULL): array {

    if (!isset($this->categoryIdNames[$langcode])) {

      $key_names = [
        static::CONF_CATEGORY_STRICTLY_NECESSARY => 'strictly necessary cookies',
        static::CONF_CATEGORY_PERFORMANCE => 'performance cookies',
        static::CONF_CATEGORY_FUNCTIONAL => 'functional cookies',
        static::CONF_CATEGORY_TARGETING => 'targeting cookies',
        static::CONF_CATEGORY_SOCIAL_MEDIA => 'social media cookies',
      ];

      // Translate with context to prevent collision with existing translations.
      foreach ($key_names as $key => $name) {
        $this->categoryIdNames[$langcode][$key] = $this->helper->translateWithContext($name, [], $langcode);
      }
    }

    return $this->categoryIdNames[$langcode];
  }

  /**
   * {@inheritdoc}
   */
  public function getConfig(): ImmutableConfig {
    return $this->config;
  }

  /**
   * {@inheritdoc}
   */
  public function getDomain(): string {

    if (!isset($this->domain)) {

      $conf_domain = $this->config->get(static::CONF_DOMAIN);
      if (is_null($conf_domain)) {
        $conf_domain = static::CONF_DOMAIN_COOKIEPRO;
      }

      $mapping = [
        static::CONF_DOMAIN_COOKIEPRO => static::SCRIPT_DOMAIN_COOKIEPRO,
        static::CONF_DOMAIN_ONETRUST => static::SCRIPT_DOMAIN_ONETRUST,
      ];

      $this->domain = $mapping[$conf_domain];
    }

    return $this->domain;
  }

  /**
   * {@inheritdoc}
   */
  public function getDomainScriptId(): string {

    if (!isset($this->domainScript)) {
      $domain_script = $this->config->get(static::CONF_DOMAIN_SCRIPT_ID);

      // Allow other modules to alter the active domain script.
      $event = new CookieProGetDomainScript($domain_script);
      $this->eventDispatcher->dispatch($event);
      $domain_script = $event->getDomainScript();

      if (empty($domain_script)) {
        $this->logger->critical('Domain Script is empty.');
      }
      elseif ($this->hasTestCdnEnabled()) {
        $domain_script .= '-test';
      }

      $this->domainScript = $domain_script;
    }

    return $this->domainScript;
  }

  /**
   * {@inheritdoc}
   */
  public function getGoogleConsentModeSupportScript(): ?string {

    if (!$this->hasGoogleConsentModeEnabled()) {
      return NULL;
    }

    $storages = $this->config->get(static::GCM_DENY_STORAGES);
    // Convert the denied storages for inclusion in inline script.
    foreach ($storages as $key => $deny) {
      if ($deny === TRUE) {
        $storages[$key] = 'denied';
      }
      else {
        unset($storages[$key]);
      }
    }

    // Set desired template.
    $script_build = [
      '#theme' => 'cookiepro_google_consent_mode',
      '#storages' => $storages,
    ];

    return $this->renderer->renderPlain($script_build);
  }

  /**
   * {@inheritdoc}
   */
  public function getCookieProHelper(): CookieProHelperInterface {
    return $this->helper;
  }

  /**
   * {@inheritdoc}
   */
  public function getIpWhitelist(): array {
    if (!isset($this->ipv4Whitelist)) {
      $this->ipv4Whitelist = ConfigHelper::configTextToArray($this->config->get(static::CONF_IP_WHITELIST));
    }

    return $this->ipv4Whitelist;
  }

  /**
   * {@inheritdoc}
   */
  public function getLogger(): LoggerInterface {
    return $this->logger;
  }

  /**
   * {@inheritdoc}
   */
  public function getMainScriptUrl(): string {
    return $this->getDomain() . static::SCRIPT_PATH;
  }

  /**
   * {@inheritdoc}
   */
  public function hasAutoBlockingEnabled(): bool {
    return (bool) $this->config->get(static::CONF_AUTO_BLOCK);
  }

  /**
   * {@inheritdoc}
   */
  public function hasGoogleConsentModeEnabled(): bool {
    return (bool) $this->config->get(static::GCM_ENABLE);
  }

  /**
   * {@inheritdoc}
   */
  public function hasLanguageDetectionEnabled(): bool {
    return (bool) $this->config->get(static::CONF_DOCUMENT_LANGUAGE);
  }

  /**
   * {@inheritdoc}
   */
  public function hasTestCdnEnabled(): bool {
    return (bool) $this->config->get(static::CONF_TEST_CDN);
  }

  /**
   * {@inheritdoc}
   */
  public function isAdminRoute(): bool {
    return $this->helper->isAdminRoute();
  }

  /**
   * {@inheritdoc}
   */
  public function isExcludedPath(): bool {

    if (!isset($this->isExcludedPath)) {

      $exclude_paths = $this->config->get(static::CONF_EXCLUDE_PATHS);
      $this->isExcludedPath = $this->helper->matchPath($this->requestStack->getCurrentRequest(), $exclude_paths);
    }

    return $this->isExcludedPath;
  }

  /**
   * {@inheritdoc}
   */
  public function isLimitedToPath(): bool {

    if (!isset($this->isLimitedToPath)) {

      $enabled = (bool) $this->config->get(static::CONF_ENABLE_LIMIT_TO_PATHS) ?? FALSE;
      if ($enabled === TRUE) {

        $limited_to_paths = $this->config->get(static::CONF_LIMIT_TO_PATHS);
        $this->isLimitedToPath = $this->helper->matchPath($this->requestStack->getCurrentRequest(), $limited_to_paths);
      }
      else {

        // The limited to paths option is disabled, all paths are allowed.
        $this->isLimitedToPath = TRUE;
      }
    }

    return $this->isLimitedToPath;
  }

  /**
   * {@inheritdoc}
   */
  public function isPaused(): bool {
    return $this->state->get(static::STATE_PAUSED, FALSE);
  }

  /**
   * {@inheritdoc}
   */
  public function isWhitelistedIp(Request $request = NULL): bool {

    $request_not_null = !is_null($request);
    if (!isset($this->isWhitelistedIp) || $request_not_null) {

      $ip = ($request_not_null) ? $request->getClientIp() : $this->requestStack->getCurrentRequest()->getClientIp();
      $match = $this->helper->matchIp($ip, $this->getIpWhitelist());

      // Don't store request specific evaluation result.
      if ($request_not_null) {
        return $match;
      }

      $this->isWhitelistedIp = $match;
    }

    return $this->isWhitelistedIp;
  }

  /**
   * {@inheritdoc}
   */
  public function mustProcess(): bool {
    return (
      !$this->isAdminRoute() &&
      !$this->isWhitelistedIp() &&
      $this->isLimitedToPath() &&
      !$this->isExcludedPath() &&
      !$this->isPaused()
    );
  }

  /**
   * {@inheritdoc}
   */
  public function testScriptUrls(): array {

    $script_id = $this->getDomainScriptId();
    $params = [
      '@environment' => $this->hasTestCdnEnabled() ? $this->helper->translateWithContext('Testing') : $this->helper->translateWithContext('Production'),
      '@script_id' => $script_id,
    ];

    $urls = [
      static::CONF_DOMAIN_SCRIPT_ID => $this->getDomain() . sprintf(static::SCRIPT_PATH_JSON, $script_id, $script_id),
    ];
    $error_messages = [
      static::CONF_DOMAIN_SCRIPT_ID => $this->helper->translateWithContext("Script for @script_id could not be loaded. Is it published to @environment CDN?", $params),
    ];

    if ($this->hasAutoBlockingEnabled()) {
      $urls[static::CONF_AUTO_BLOCK] = $this->getAutoBlockScriptUrl();
      $error_messages[static::CONF_AUTO_BLOCK] = $this->helper->translateWithContext("Auto-Blocking™ script could not be loaded. Is it enabled and published to @environment CDN?", $params);
    }

    $errors = $this->helper->testUrls($urls);
    foreach ($errors as $key => $error) {
      $context = [
        'url' => $urls[$key],
        'error' => $error,
      ];
      $this->logger->error("Error when fetching {url}: {error} ", $context);
      $errors[$key] = $error_messages[$key];
    }

    return $errors;
  }

  /**
   * Set pause mode status.
   *
   * @param bool $enabled
   *   TRUE to enable the pause modus.
   */
  private function setPauseMode(bool $enabled = FALSE): void {

    $this->state->set(static::STATE_PAUSED, $enabled);
    Cache::invalidateTags($this->config->getCacheTags());
  }

}
